Option Explicit
'================================================================================================
'Name: ZonePlayer Control Script
'Author: Osler
'Info: A continuously running script to instantiate a Sonos ZonePlayer as a HouseBot device. This
'      script uses UPnP callback functionality to minimize network traffic. The ZonePlayer will
'      not be polled for its status unless polling is explicitly called for.
'Required Device Properties:
'			UDN					(alphanumeric)
'			ZPName				(alphanumeric)
'			ZPIPAddress			(alphanumeric)
'			ZPTransportState	(alphanumeric)	(Play, Pause, Stop, Previous, Next)
'			ZPPlayMode			(alphanumeric)	(Normal, Shuffle, Shuffle No Repeat, Repeat All)
'			ZPVolume			(alphanumeric)	(0 - 100)
'			ZPBass				(alphanumeric)	(-10 - 10)
'			ZPTreble			(alphanumeric)	(-10 - 10)
'			ZPMuteStatus		(boolean)
'			ZPLoudnessStatus	(boolean)
'			ZPCurrentArtist		(alphanumeric)
'			ZPCurrentAlbum		(alphanumeric)
'			ZPCurrentTrack		(alphanumeric)
'			ZPTrackLength		(alphanumeric)
'			ZPTrackPosition		(alphanumeric)
'			ZPTrackPosLen		(alphanumeric)
'			ZPTrackPercent		(alphanumeric)
'			ZPRemoveURI			(alphanumeric)	(Populate via property substitution from ZPQueueSelection)
'			ZPAddURI			(alphanumeric)	(Populate via property substitution from ZPBrowseSelection)
'			ZPPlayNowURI		(alphanumeric)	(Populate via property substitution from ZPBrowseSelection)
'			ZPPlayNextURI		(alphanumeric)	(Populate via property substitution from ZPBrowseSelection)
'			ZPNowPlayingPath	(alphanumeric)	(Path for album art of currently playing track)
'			ZPNextPlayingPath	(alphanumeric)	(Path for album art of next playing track)
'			ZPMusicMenuTitle	(alphanumeric)
'			ZPClearPlayURI		(alphanumeric)
'			ZPQueue				(alphalist)
'			ZPQueuePosition		(alphanumeric)
'			ZPQueueMoveUp		(alphanumeric)
'			ZPQueueMoveDown		(alphanumeric)
'			ZPClearQueue		(boolean)
'			ZPQueueSize			(alphanumeric)
'			ZPQueuePosSize		(alphanumeric)
'			ZPQueueSelection	(alphanumeric)	(Linked to Queue)
'			ZPBrowse			(alphanumeric)	(Forward, Back, Up, Down, Hold)
'			ZPBrowseIndex		(alphanumeric)
'			ZPBrowseList		(alphalist)
'			ZPBrowseTotalItems	(alphanumeric)
'			ZPBrowseSelection	(alphanumeric)	(Linked to BrowseList)
'			ZPNextTrack			(alphanumeric)
'			ZPGroupCoordinator	(alphanumeric)
'			ZPDisplay1			(alphanumeric)
'			ZPDisplay2			(alphanumeric)
'			ZPDisplay3			(alphanumeric)
'			ZPDisplay4			(alphanumeric)
'			ZPLinked			(boolean)
'			ZPUnlink			(boolean)
'			ZPBrowsePercent		(alphanumeric)
'			ZPRampToVolume		(alphanumeric)
'			ZPRestorePreRampVolume	(boolean)
'			ZPReindexMusic		(boolean)
'			ZPIndexType			(alphanumeric)
'			ZPSearchType 		(alphanumeric) (A:ALBUM, A:ARTIST, A:TRACKS, A:GENRE, A:COMPOSER, A:PLAYLISTS, R:, AI:, S:, SQ:)
'			ZPSearchString		(alphanumeric) (#, A-Z (upper case); single character)
'			ZPShuffle			(boolean)
'			ZPRepeat			(boolean)
'			ZPForcedResubscription	(boolean)
'			ZPResubscriptionInterval	(alphanumeric)
'			ZPLinkedZones		(alphanumeric)
'==========================================================================================================
' Current ZonePlayer Status Variables Obtained from the ZonePlayer
'==========================================================================================================
Dim ZonePlayer
Dim CurrentTransportState
Dim CurrentPlayMode
Dim CurrentVolumeLevel
Dim CurrentTrebleLevel
Dim CurrentBassLevel
Dim CurrentMuteStatus
Dim CurrentLoudnessStatus
Dim CurrentURI
Dim CurrentTrack
Dim CurrentArtist
Dim CurrentAlbum
Dim CurrentTrackLength
Dim CurrentTrackPosition
Dim CurrentTrackPercent
Dim CurrentQueue
Dim CurrentQueuePosition
Dim CurrentQueueSize
'==========================================================================================================
' Input ZonePlayer Command Variables Obtained from the HouseBot Device
'==========================================================================================================
Dim NewTransportState
Dim NewPlayMode
Dim NewVolumeLevel
Dim NewTrebleLevel
Dim NewBassLevel
Dim NewMuteStatus
Dim NewLoudnessStatus
Dim RemoveURI
Dim AddURI
Dim PlayNowURI
Dim PlayNextURI
Dim ClearPlayURI
Dim ClearQueue
Dim UDN
Dim GroupCoordinator
Dim DeviceCoordinator
Dim Connection
Dim IPAddress
Dim ZoneName
Dim StoredQueue
Dim StoredQueuePosition
Dim StoredQueueSize
Dim UpdateQueue
Dim Browse
Dim BrowseIndex
Dim BrowseReturn
Dim BrowseSelection
Dim QueueSelection
Dim RampVolume
Dim RestoreVolume
Dim ReindexMusic
Dim IndexType
Dim SearchType
Dim SearchIndex
Dim NewShuffle
Dim NewRepeat
Dim MusicSelectionList
'==========================================================================================================
' Saved Values
'==========================================================================================================
Dim SavedTransportState
Dim SavedPlayMode
Dim SavedVolumeLevel
Dim SavedTrebleLevel
Dim SavedBassLevel
Dim SavedMuteStatus
Dim SavedURI
Dim SavedLoudnessStatus
Dim SavedBrowseSelection(5)
Dim SavedBrowseIndex(5)
Dim SavedBrowseList(5)
Dim SavedBrowseTotalItems(5)
Dim BrowsePercent
Dim MusicMenuTitle(5)
Dim BrowseDepth
Dim RadioCatalog
Dim NowPlayingPath
Dim NextPlayingPath
Dim NoAlbumArt
Dim RadioArt
Dim TransparentImage
Dim Zones()
Dim Unlink
Dim Reachable
Dim Online
Dim CallbackTime
Dim SavedSearchType
Dim SavedShuffle
Dim SavedRepeat
Dim SavedSelection
'==========================================================================================================
' Flags
'==========================================================================================================
Dim TransportUpdate
Dim RenderingUpdate
Dim Update(6)
Dim Radio
Dim TrackTime
'==========================================================================================================
' MAIN
'==========================================================================================================
ZonePlayerConnect
If ZonePlayer.Reachable = True Then
	GetZonePlayerStateActively
	GetTrackInfo
End If
GetOtherZP_Devices

Do
	If GetPropertyValue(".ZPScriptState") = 1 Then Exit Do
	Main
	Sleep(GetPropertyValue(".ZPScriptInterval"))
Loop

Call TraceMessage("The script has been stopped.")
Call SetPropertyValue(".ZPScriptState", 0)
ZonePlayerDisconnect

Private Sub Main
	WriteToLog
	GetDeviceValues
	CompareNewToSaved										
	GetZonePlayerStatePassively
	UpdateLinkedZones
	SynchronizeData
	BrowseMedia2
	Housekeeping
	If GetPropertyValue(".ZPTransportState") = "Play" Then
		GetTrackInfo
	End If
	If DateAdd("n", CInt(GetPropertyValue(".ZPResubscriptionInterval")), CallbackTime) < Now Then
		CallbackTime = Now
		If ZonePlayer.Reachable = True Then
			TraceMessage("A callback has not been received for " & CStr(GetPropertyValue(".ZPResubscriptionInterval")) & " minutes but the ZonePlayer responded to a ping.")
			If GetPropertyValue(".ZPForcedResubscription") = 1 Then
				ZonePlayerDisconnect
				ZonePlayerConnect
			End If
		Else
			TraceMessage("A callback has not been received for " & CStr(GetPropertyValue(".ZPResubscriptionInterval")) & " minutes and the ZonePlayer did not respond to a ping.")
			ZonePlayerDisconnect
			ZonePlayerConnect
		End If
	End If
End Sub


'==========================================================================================================
' Group Coordination (Linking)
'==========================================================================================================
Private Function UpdateGroupCoordinator()
	On Error Resume Next
	GroupCoordinator = ZonePlayer.GetGroupCoordinator()
	If GroupCoordinator = "Offline" Then
		UpdateGroupCoordinator = "Offline"
		Exit Function
	End If
	If Mid(GroupCoordinator, 1, 4) = "uuid" Then
		Dim I
		For I = 0 To UBound(Zones)
			If GroupCoordinator = Zones(I, 1) Then
				DeviceCoordinator = Zones(I, 0)
				Call SetPropertyValue(".ZPGroupCoordinator", Replace(Zones(I, 0), "ZP_", ""))
				If Replace(Zones(I, 0), "ZP_", "") <> ZoneName Then
					Call SetPropertyValue(".ZPLinked", 1)
				Else
					Call SetPropertyValue(".ZPLinked", 0)
				End If
				Exit For
			End If
		Next
	End If
	If Err <> 0 Then
		TraceMessage("UpdateGroupCoordinator Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	UpdateGroupCoordinator = "OK"
End Function

Private Sub UpdateLinkedZones()
	Dim LinkedZones
	Dim I
	LinkedZones = ""
	If GetPropertyValue(".ZPLinked") = 0 Then
		For I = 0 To UBound(Zones)
			If Replace(Zones(I, 0), "ZP_", "") <> ZoneName And Zones(I, 0) <> "" Then
				If GetPropertyValue(Zones(I, 0) & ".ZPGroupCoordinator") = ZoneName Then
					LinkedZones = LinkedZones & "+ " & Replace(Zones(I, 0), "ZP_", "") & " "
				End If
			End If
		Next
		If GetPropertyValue(".ZPLinkedZones") <> LinkedZones Then
			Call SetPropertyValue(".ZPLinkedZones", LinkedZones)
		End If
	End If
End Sub

Private Sub GetOtherZP_Devices()
	On Error Resume Next
	Dim HBDevices
	Dim ZP_DeviceList
	Dim ZP_Devices
	Dim I
	HBDevices = Split(GetDeviceList(Chr(149)), Chr(149))
	For I = 0 To UBound(HBDevices)
		If Mid(HBDevices(I), 1, 3) = "ZP_" Then
			ZP_DeviceList = ZP_DeviceList & HBDevices(I) & Chr(149)
		End If
	Next
	ZP_Devices = Split(ZP_DeviceList, Chr(149))
	ReDim Zones(UBound(ZP_Devices), 1)
	For I = 0 To UBound(ZP_Devices)
		Zones(I, 0) = ZP_Devices(I)
		Zones(I, 1) = GetPropertyValue(ZP_Devices(I) & ".UDN")
	Next
	If Err <> 0 Then
		TraceMessage("GetOtherZP_Devices Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub
'==========================================================================================================
' Tracing
'==========================================================================================================
Private Sub WriteToLog()
	On Error Resume Next
    Dim Buffer
    Dim SplitBuffer
    Buffer = ZonePlayer.GetMessageBuffer
    If IsArray(Buffer) = True or Buffer <> "" Then
         Call ZonePlayer.ResetMessageBuffer
         If InStr(Buffer, Chr(149)) <> 0 Then
             SplitBuffer = Split(Buffer, Chr(149))
             Dim FNSB
             For FNSB = 0 To UBound(SplitBuffer)
                 TraceMessage(SplitBuffer(FNSB))
             Next
	     Call ZonePlayer.ResetMessageBuffer
         Else
             TraceMessage(Buffer)
	     Call ZonePlayer.ResetMessageBuffer
         End If
     End If
     If Err <> 0 Then
		TraceMessage("WriteToLog Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub
'==========================================================================================================
' ZonePlayer Connection and Disconnection Subroutines
'==========================================================================================================
Private Sub ZonePlayerConnect()
	On Error Resume Next
	UDN = GetPropertyValue(".UDN")
	Set ZonePlayer = CreateObject("ZonePlayer.Controller")
	Connection = ZonePlayer.Connect(UDN)
	Do Until ZonePlayer.Online = True
		Sleep(20)
	Loop
	ZoneName = ZonePlayer.GetZoneName
	Call SetPropertyValue(".ZPName", ZoneName)
	Sleep(250)
	IPAddress = ZonePlayer.GetIPAddress
	Call SetPropertyValue(".ZPIPAddress", IPAddress)
	Sleep(250)
	RadioCatalog = ZonePlayer.RadioStationCatalog
		
	BrowseDepth = 0
	Call SetPropertyValue(".ZPBrowseList", SavedBrowseList(0))
	Call SetPropertyValue(".ZPBrowseIndex", "0")
	Call SetPropertyValue(".ZPQueuePosition", "0")
	Call SetPropertyValue(".ZPMusicMenuTitle", "Music Menu")
	Call SetPropertyValue(".ZPAddURI", "")
	Call SetPropertyValue(".ZPRemoveURI", "")
	Call SetPropertyValue(".ZPPlayNextURI", "")
	Call SetPropertyValue(".ZPPlayNowURI", "")
	Call SetPropertyValue(".ZPClearPlayURI", "")
	Call SetPropertyValue(".ZPQueue", "" & vbTab & "" & vbLf)
	Call SetPropertyValue(".ZPClearQueue", 0)
	Call SetPropertyValue(".ZPSearchType", "A:ALBUM")
	Call SetPropertyValue(".ZPBrowseDepth", 0)
	CurrentQueuePosition = "0"
	CurrentQueueSize = "0"
	Call SetPropertyValue(".ZPQueueSize", "1")
	NowPlayingPath = "C:\Program Files\HouseBot\Config\Themes\Sonos Images\" & ZoneName & " NowPlaying.jpg"
	NextPlayingPath = "C:\Program Files\HouseBot\Config\Themes\Sonos Images\" & ZoneName & " NextPlaying.jpg"
	NoAlbumArt = "C:\Program Files\HouseBot\Config\Themes\Sonos Images\No Album Art.png"
	RadioArt = "C:\Program Files\HouseBot\Config\Themes\Sonos Images\Internet Radio.png"
	TransparentImage = "C:\Program Files\HouseBot\Config\Themes\Sonos Images\TransparentImage.jpg"
	If Err <> 0 Then
		TraceMessage("ZonePlayerConnect Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub

Private Sub ZonePlayerDisconnect()
	ZonePlayer.Disconnect
	Set ZonePlayer = Nothing
	TraceMessage("Connection to ZonePlayer: " & ZoneName & " (" & IPAddress & ") terminated.")
End Sub	
'==========================================================================================================
' Initialize ZonePlayer Command Variables with HouseBot Device Values
'==========================================================================================================
Private Sub GetDeviceValues()
	On Error Resume Next
	NewTransportState = GetPropertyValue(".ZPTransportState")
	NewPlayMode = GetPropertyValue(".ZPPlayMode")
	NewVolumeLevel = GetPropertyValue(".ZPVolume")
	NewTrebleLevel = GetPropertyValue(".ZPTreble")
	NewBassLevel = GetPropertyValue(".ZPBass")
	NewMuteStatus = CStr(CBool(GetPropertyValue(".ZPMuteStatus")))
	NewLoudnessStatus = CStr(CBool(GetPropertyValue(".ZPLoudnessStatus")))
	StoredQueue = GetPropertyValue(".ZPQueue")
	StoredQueuePosition = GetPropertyValue(".ZPQueuePosition")
	StoredQueueSize = GetPropertyValue(".ZPQueueSize")
	UpdateQueue = GetPropertyValue(".ZPUpdateQueue")
	RemoveURI = GetPropertyValue(".ZPRemoveURI")
	AddURI = GetPropertyValue(".ZPAddURI")
	PlayNowURI = GetPropertyValue(".ZPPlayNowURI")
	PlayNextURI = GetPropertyValue(".ZPPlayNextURI")
	ClearPlayURI = GetPropertyValue(".ZPClearPlayURI")
	ClearQueue = CStr(CBool(GetPropertyValue(".ZPClearQueue")))
	QueueSelection = GetPropertyValue(".ZPQueueSelection")
	Browse = GetPropertyValue(".ZPBrowse")
	If Browse <> "Hold" Then
		Call SetPropertyValue(".ZPBrowse", "Hold")
	End If
	BrowseIndex = CStr(GetPropertyValue(".ZPBrowseIndex"))
	BrowseReturn = CStr(GetPropertyValue(".ZPBrowseReturn"))
	BrowseSelection = CStr(GetPropertyValue(".ZPBrowseSelection"))
	Unlink = CStr(CBool(GetPropertyValue(".ZPUnlink")))
	RampVolume = GetPropertyValue(".ZPRampToVolume")
	RestoreVolume = CStr(CBool(GetPropertyValue(".ZPRestorePreRampVolume")))
	ReindexMusic = CStr(CBool(GetPropertyValue(".ZPReindexMusic")))
	IndexType = GetPropertyValue(".ZPIndexType")
	SearchType = GetPropertyValue(".ZPSearchType")
	SearchIndex = GetPropertyValue(".ZPSearchString")
	NewShuffle = GetPropertyValue(".ZPShuffle")
	NewRepeat = GetPropertyValue(".ZPRepeat")
	If Err <> 0 Then
		TraceMessage("GetDeviceValues Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub
'==========================================================================================================
' Synchronize the ZonePlayer and Housebot
'==========================================================================================================
Private Sub SynchronizeData()
	On Error Resume Next
	Dim I
	
	If CurrentTransportState <> NewTransportState Then						
		If TransportUpdate = True Then								
			If Update(0) = False Then				
				SavedTransportState = CurrentTransportState					
				Call SetPropertyValue(".ZPTransportState", SavedTransportState)	
			Else
				Call ZonePlayer.SetTransportState(NewTransportState)	
			End If
		Else
			If Update(0) = False Then
				Call ZonePlayer.SetTransportState(NewTransportState)
			End If																					
		End If																	
	End If
	
	If Err <> 0 Then
		TraceMessage("SynchronizeTransport Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
																				
	If CInt(CurrentVolumeLevel) <> CInt(NewVolumeLevel) Then								'
		If RenderingUpdate = True Then
			If Update(2) = False Then
				SavedVolumeLevel = CurrentVolumeLevel
				Call SetPropertyValue(".ZPVolume", SavedVolumeLevel)
			Else
				Call ZonePlayer.SetVolumeLevel("Master", NewVolumeLevel)
			End If
		Else
			If Update(2) = False Then
				Call ZonePlayer.SetVolumeLevel("Master", NewVolumeLevel)
			End If
		End If
	End If
	
	If Err <> 0 Then
		TraceMessage("SynchronizeVolume Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If CurrentTrebleLevel <> NewTrebleLevel Then
		If RenderingUpdate = True Then
			If Update(3) = False Then
				SavedTrebleLevel = CurrentTrebleLevel
				Call SetPropertyValue(".ZPTreble", SavedTrebleLevel)
			Else
				Call ZonePlayer.SetTrebleLevel(CInt(NewTrebleLevel))
			End If
		Else
			If Update(3) = False Then
				Call ZonePlayer.SetTrebleLevel(CInt(NewTrebleLevel))
			End If
		End If
	End If
	
	If Err <> 0 Then
		TraceMessage("SynchronizeTreble Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If CurrentBassLevel <> NewBassLevel Then
		If RenderingUpdate = True Then
			If Update(4) = False Then
				SavedBassLevel = CurrentBassLevel
				Call SetPropertyValue(".ZPBass", SavedBassLevel)
			Else
				Call ZonePlayer.SetBassLevel(CInt(NewBassLevel))
			End If
		Else
			If Update(4) = False Then
				Call ZonePlayer.SetBassLevel(CInt(NewBassLevel))
			End If
		End If
	End If
	
	If Err <> 0 Then
		TraceMessage("SynchronizeBass Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If CurrentMuteStatus <> NewMuteStatus Then
		If RenderingUpdate = True Then
			If Update(5) = False Then
				SavedMuteStatus = CurrentMuteStatus
				If SavedMuteStatus = "True" Then
					Call SetPropertyValue(".ZPMuteStatus", 1)
				ElseIf SavedMuteStatus = "False" Then
					Call SetPropertyValue(".ZPMuteStatus", 0)
				End If
			Else
				If NewMuteStatus = "True" Then
					Call ZonePlayer.SetMuteState("Master", True)
				ElseIf NewMuteStatus = "False" Then
					Call ZonePlayer.SetMuteState("Master", False)
				End If
			End If
		Else
			If Update(5) = False Then
				If NewMuteStatus = "True" Then
					Call ZonePlayer.SetMuteState("Master", True)
				ElseIf NewMuteStatus = "False" Then
					Call ZonePlayer.SetMuteState("Master", False)
				End If
			End If
		End If
	End If
	
	If Err <> 0 Then
		TraceMessage("SynchronizeMute Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If CurrentLoudnessStatus <> NewLoudnessStatus Then
		If RenderingUpdate = True Then
			If Update(6) = False Then
				SavedLoudnessStatus = CurrentLoudnessStatus
				If SavedLoudnessStatus = "True" Then
					Call SetPropertyValue(".ZPLoudnessStatus", 1)
				ElseIf SavedLoudnessStatus = "False" Then
					Call SetPropertyValue(".ZPLoudnessStatus", 0)
				End If
			Else
				If NewLoudnessStatus = "True" Then
					Call ZonePlayer.SetLoudnessState("Master", True)
				ElseIf NewMuteStatus = "False" Then
					Call ZonePlayer.SetLoudnessState("Master", False)
				End If
			End If
		Else
			If Update(6) = False Then
				If NewLoudnessStatus = "True" Then
					Call ZonePlayer.SetLoudnessState("Master", True)
				ElseIf NewMuteStatus = "False" Then
					Call ZonePlayer.SetLoudnessState("Master", False)
				End If
			End If
		End If
	End If
	
	If Err <> 0 Then
		TraceMessage("SynchronizeLoudness Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	For I = 0 To 6															
		Update(I) = False
	Next
	TransportUpdate = False
	RenderingUpdate = False
End Sub

Private Sub Housekeeping()
	On Error Resume Next
	Dim ItemID
	Dim Meta
	Dim I
	Dim CurrentPosition
	Dim EnqueuePosition
	Dim RadioSplit
	Dim BS
	Dim BSSplit
	
	If Unlink = "True" Then
		Call ZonePlayer.Unlink
		Call SetPropertyValue(".ZPUnlink", 0)
	End If
	
	If Err <> 0 Then
		TraceMessage("Unlink Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If

	If ClearPlayURI <> "" Then
		If Mid(ClearPlayURI, 1, 17) = "x-rincon-mp3radio" Then
			RadioSplit = Split(ClearPlayURI, Chr(149))
			ClearPlayURI = RadioSplit(0)
			CurrentTrack = RadioSplit(1)
			ItemID = RadioSplit(2)
			Call ZonePlayer.ClearQueue
			Meta = ZonePlayer.GetObjectMetadata(ItemID)
			Call ZonePlayer.PlaySongNow(ClearPlayURI, Meta)
			SavedTransportState = "Play"
			Call ZonePlayer.SetTransportState(SavedTransportState)
			Call SetPropertyValue(".ZPTransportState", SavedTransportState)
		ElseIf Mid(ClearPlayURI, 1, 1) = "x" or Mid(ClearPlayURI, 1, 1) = "f" Then
			Call ZonePlayer.ClearQueue
			Meta = ZonePlayer.GetObjectMetadata(ClearPlayURI)
			Call ZonePlayer.AddSongToQueue(ClearPlayURI, Meta, 0, False)
			Sleep(250)
			Call ZonePlayer.PlaySongNow("x-rincon-queue:" & Replace(UDN, "uuid:", "") & "#0", "")
		End IF
		ClearPlayURI = ""
		Call SetPropertyValue(".ZPClearPlayURI", ClearPlayURI)
		Call SetPropertyValue(".ZPTransportState", "Play")
		UpdateQueue = "True"
	End If
	
	If Err <> 0 Then
		TraceMessage("ClearPlayURI Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If RemoveURI <> "" And RemoveURI <> "-1" Then													
		Call ZonePlayer.RemoveTrackFromQueue(RemoveURI)
		RemoveURI = ""
		Call SetPropertyValue(".ZPRemoveURI", RemoveURI)
		UpdateQueue = "True"
	End If
	
	If Err <> 0 Then
		TraceMessage("RemoveURI Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If AddURI <> "" Then													
		If Mid(AddURI, 1, 17) = "x-rincon-mp3radio" Then
			RadioSplit = Split(AddURI, Chr(149))
			AddURI = RadioSplit(0)
			CurrentTrack = RadioSplit(1)
			ItemID = RadioSplit(2)
			Meta = ZonePlayer.GetObjectMetadata(ItemID)
			Call ZonePlayer.PlaySongNow(AddURI, Meta)
			SavedTransportState = "Play"
			Call ZonePlayer.SetTransportState(SavedTransportState)
			Call SetPropertyValue(".ZPTransportState", SavedTransportState)
		ElseIf Mid(AddURI, 1, 1) = "x" or Mid(AddURI, 1, 1) = "f" Then
			Meta = ZonePlayer.GetObjectMetadata(AddURI)
			If CInt(CurrentQueueSize) = 0 Or Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Then
				Call ZonePlayer.AddSongToQueue(AddURI, Meta, 0, False)
				Sleep(250)
				Call ZonePlayer.PlaySongNow("x-rincon-queue:" & Replace(UDN, "uuid:", "") & "#0", "")
			Else
				Call ZonePlayer.AddSongToQueue(AddURI, Meta, 0, False)
			End If 
		End IF
		AddURI = ""
		Call SetPropertyValue(".ZPAddURI", AddURI)
		UpdateQueue = "True"
	End If
	
	If Err <> 0 Then
		TraceMessage("AddURI Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If PlayNextURI <> "" Then													
		If Mid(PlayNextURI, 1, 17) = "x-rincon-mp3radio" Then
			RadioSplit = Split(PlayNextURI, Chr(149))
			PlayNextURI = RadioSplit(0)
			CurrentTrack = RadioSplit(1)
			ItemID = RadioSplit(2)
			Meta = ZonePlayer.GetObjectMetadata(ItemID)
			Call ZonePlayer.PlaySongNow(PlayNextURI, Meta)
			SavedTransportState = "Play"
			Call ZonePlayer.SetTransportState(SavedTransportState)
			Call SetPropertyValue(".ZPTransportState", SavedTransportState)
		ElseIf Mid(PlayNextURI, 1, 1) = "x" or Mid(PlayNextURI, 1, 1) = "f" Then
			Meta = ZonePlayer.GetObjectMetadata(PlayNextURI)
			If CInt(CurrentQueueSize) = 0 Or Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Then
				Call ZonePlayer.AddSongToQueue(PlayNextURI, Meta, 0, False)
				Sleep(250)
				Call ZonePlayer.PlaySongNow("x-rincon-queue:" & Replace(UDN, "uuid:", "") & "#0", "")
			Else
				Call ZonePlayer.AddSongToQueue(PlayNextURI, Meta, 0, True)
				Sleep(250)
				CurrentPosition = CInt(CurrentQueueSize) + 1
				EnqueuePosition = CInt(CurrentQueuePosition) + 1
				Call ZonePlayer.ChangeTrackPosition(CurrentPosition, EnqueuePosition)
			End If
			Radio = False
		ElseIf Mid(PlayNextURI, 1, 3) = "Q:0" Then
			PlayNextURI = Replace(PlayNextURI, "Q:0/", "")
			EnqueuePosition = CInt(CurrentQueuePosition) + 1
			Call ZonePlayer.ChangeTrackPosition(CInt(PlayNextURI), EnqueuePosition)
		End IF
		If IsNumeric(PlayNextURI) = True Then
			If CInt(PlayNextURI) < EnqueuePosition Then
				UpdateQueue = "False"
			Else
				UpdateQueue = "True"
			End If
		Else
			UpdateQueue = "True"
		End If
		PlayNextURI = ""
		Call SetPropertyValue(".ZPPlayNextURI", PlayNextURI)
	End If
	
	If Err <> 0 Then
		TraceMessage("PlayNextURI Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If PlayNowURI <> "" Then							
		If Mid(PlayNowURI, 1, 17) = "x-rincon-mp3radio" Then
			RadioSplit = Split(PlayNowURI, Chr(149))
			PlayNowURI = RadioSplit(0)
			CurrentTrack = RadioSplit(1)
			ItemID = RadioSplit(2)
			Meta = ZonePlayer.GetObjectMetadata(ItemID)
			Call ZonePlayer.PlaySongNow(PlayNowURI, Meta)
			SavedTransportState = "Play"
			Call ZonePlayer.SetTransportState(SavedTransportState)
			Call SetPropertyValue(".ZPTransportState", SavedTransportState)
		ElseIf Mid(PlayNowURI, 1, 1) = "x" or Mid(PlayNowURI, 1, 1) = "f" Then
			If Mid(PlayNowURI, 1, 9) = "x-rincon:" Or Mid(PlayNowURI, 1, 16) = "x-rincon-stream:"Then
				Call ZonePlayer.PlaySongNow(PlayNowURI, "")
			Else
				Meta = ZonePlayer.GetObjectMetadata(PlayNowURI)
				If CInt(CurrentQueueSize) = 0 Or Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Then
					Call ZonePlayer.AddSongToQueue(PlayNowURI, Meta, 0, False)
					Sleep(250)
					Call ZonePlayer.PlaySongNow("x-rincon-queue:" & Replace(UDN, "uuid:", "") & "#0", "")
				Else
					Call ZonePlayer.AddSongToQueue(PlayNowURI, Meta, 0, False)
					Sleep(250)
					CurrentPosition = CInt(CurrentQueueSize) + 1
					Call ZonePlayer.SeekTrack(CurrentPosition)
				End If
			End If
			Radio = False
		ElseIf Mid(PlayNowURI, 1, 3) = "Q:0" Then
			If Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Or Mid(CurrentURI, 1, 15) = "x-rincon-stream" Then
				Call ZonePlayer.PlaySongNow("x-rincon-queue:" & Replace(UDN, "uuid:", "") & "#0", "")
			End If
			PlayNowURI = Replace(PlayNowURI, "Q:0/", "")
			Call ZonePlayer.SeekTrack(CInt(PlayNowURI))
		End IF
		PlayNowURI = ""
		Call SetPropertyValue(".ZPPlayNowURI", PlayNowURI)
		Call SetPropertyValue(".ZPTransportState", "Play")
	End If
	
	If Err <> 0 Then
		TraceMessage("PlayNowURI Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If

	If RampVolume <> "" Then
		If IsNumeric(RampTime) = True Then
			Call ZonePlayer.RampToVolume("Master", CInt(RampVolume))
		End If
		Call SetPropertyValue(".ZPRampToVolume", "")
	End If

	If Err <> 0 Then
		TraceMessage("RampToVolume Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If

	If RestoreVolume = "True" Then
		Call ZonePlayer.RestoreVolumePriorToRamp("Master")
		Call SetPropertyValue(".ZPRestorePreRampVolume", 0)
	End If

	If Err <> 0 Then
		TraceMessage("RestoreVolume Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If BrowseSelection <> SavedSelection Then
		If InStr(1, BrowseSelection, "A:ALBUM") <> 0 Then
			BS = unescape(Replace(BrowseSelection, "A:ALBUM/", ""))
			If InStr(1, BS, "/") = 0 Then
				BS = Replace(BS, ":", "-")
				BS = Replace(BS, "&", "-")
				BS = Replace(BS, "<", "-")
				BS = Replace(BS, ">", "-")
				BS = Replace(BS, "<", "-")
				BS = Replace(BS, "\", "-")
				BS = Replace(BS, "/", "-")
				BS = Replace(BS, "?", "-")
				BS = Replace(BS, "*", "-")
				BS = "C:\Program Files\HouseBot\Config\Themes\SonosAlbumArt\" & BS & ".jpg"
			End If
		ElseIf InStr(1, BrowseSelection, "A:ARTIST") <> 0 Then
			BS = unescape(Replace(BrowseSelection, "A:ARTIST/", ""))
			If InStr(1, BS, "/") = 0 Then
				BS = "C:\Program Files\HouseBot\Config\Themes\SonosAlbumArt\TransparentImage.png"
			Else
				BSSplit = Split(BS, "/")
				BS = BSSplit(1)
				BS = Replace(BS, ":", "-")
				BS = Replace(BS, "&", "-")
				BS = Replace(BS, "<", "-")
				BS = Replace(BS, ">", "-")
				BS = Replace(BS, "<", "-")
				BS = Replace(BS, "\", "-")
				BS = Replace(BS, "/", "-")
				BS = Replace(BS, "?", "-")
				BS = Replace(BS, "*", "-")
				BS = "C:\Program Files\HouseBot\Config\Themes\SonosAlbumArt\" & BS & ".jpg"
			End If
		ElseIf InStr(1, BrowseSelection, "A:GENRE") <> 0 Then
			BS = unescape(Replace(BrowseSelection, "A:GENRE/", ""))
			If InStr(1, BS, "/") = 0 Then
				BS = "C:\Program Files\HouseBot\Config\Themes\SonosAlbumArt\TransparentImage.png"
			Else
				BSSplit = Split(BS, "/")
				If UBound(BSSplit) = 2 Then
					BS = BSSplit(2)
					BS = Replace(BS, ":", "-")
					BS = Replace(BS, "&", "-")
					BS = Replace(BS, "<", "-")
					BS = Replace(BS, ">", "-")
					BS = Replace(BS, "<", "-")
					BS = Replace(BS, "\", "-")
					BS = Replace(BS, "/", "-")
					BS = Replace(BS, "?", "-")
					BS = Replace(BS, "*", "-")
					BS = "C:\Program Files\HouseBot\Config\Themes\SonosAlbumArt\" & BS & ".jpg"
				Else
					BS = "C:\Program Files\HouseBot\Config\Themes\SonosAlbumArt\TransparentImage.png"
				End If
			End If
		ElseIf InStr(1, BrowseSelection, "x-file-cifs") <> 0 And SearchType = "A:TRACKS" Then
			BS = "C:\Program Files\HouseBot\Config\Themes\SonosAlbumArt\TransparentImage.jpg"
		End If
		Call SetPropertyValue(".ZPBrowseSelectionImage", BS)
		Call SetPropertyValue(".ZPPlayAllURI", "x-rincon-playlist:" & Replace(UDN, "uuid:", "") & "#" & BrowseSelection)
		SavedSelection = BrowseSelection
	End If

	If Err <> 0 Then
		TraceMessage("SelectionImage Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If

	If ReindexMusic = "True" Then
		Call ZonePlayer.ReindexMusic(IndexType)
		Call SetPropertyValue(".ZPReindexMusic", 0)
	End If

	If Err <> 0 Then
		TraceMessage("ReindexMusic Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If ClearQueue = "True" Then												
		Call ZonePlayer.ClearQueue
		ClearQueue = "False"
		Call SetPropertyValue(".ZPClearQueue", 0)
		Call ZonePlayer.PlaySongNow("", "")
		Call SetPropertyValue(".ZPNextTrack", "")
		UpdateQueue = "True"
		Call SetPropertyValue(".ZPSearchString", "UpdateQueue")
	End If
	
	If IsNumeric(StoredQueuePosition) = False Or _
		IsNumeric(CurrentQueuePosition) = False Or _
		CInt(CurrentQueueSize) <> CInt(StoredQueueSize) Or _
		CInt(CurrentQueuePosition) <> CInt(StoredQueuePosition) Or _
		UpdateQueue = "True" Then	
														
		Dim Q
		Dim QNumber
		Dim QTrack
		Dim QArtist
		Dim QAlbum
		Dim ErrorCheck	
																	
		CurrentQueue = ""												
		Q = ZonePlayer.GetQueue   
		  
		If VarType(Q) <> 8204 Then
			Q = ZonePlayer.GetQueue
			If VarType(Q) <> 8204 Then
				Q = ZonePlayer.GetQueue
				If VarType(Q) <> 8204 Then
					CurrentQueueSize = "0"
					CurrentQueue = "0" & vbTab & "<Queue Empty>" & vbLf
				End If
			End If
		End If
		
		If CurrentQueue = "" Then
			CurrentQueueSize = (UBound(Q,1) + 1)
			For I = 0 To UBound(Q,1)
				QNumber = CStr(I + 1)
				QTrack = CStr(Q(I,2))
				QArtist = CStr(Q(I,0))
				QAlbum = CStr(Q(I,1))
				If Mid(QTrack, 1, 1) = "+" Then
					Call SetPropertyValue(".ZPHHMessage", Mid(QAlbum, 2, Len(QAlbum) - 1) & " - " & Mid(QTrack, 2, Len(QTrack) - 1))
					Call SetPropertyValue(".ZPRemoveURI", Q(I, 5))
				End If
				Call StandardizeLength(QNumber, 6)
				Call StandardizeLength(QTrack, 30)
				Call StandardizeLength(QArtist, 30)
				Call StandardizeLength(QAlbum, 30)
				
				If IsNumeric(CurrentQueuePosition) = True Then
					If I = (CInt(CurrentQueuePosition) - 1) Then
						CurrentQueue = CurrentQueue & "*S-" & CStr(Q(I,5)) & _
							vbTab & "> " & _
							vbTab & QTrack & vbLf
					Else
						CurrentQueue = CurrentQueue & CStr(Q(I,5)) & _
							vbTab & "  " & _
							vbTab & QTrack & vbLf
					End If
				Else
					CurrentQueue = CurrentQueue & CStr(Q(I,5)) & _
						vbTab & "  " & _
						vbTab & QTrack & vbLf
				End If
				If Len(CurrentQueue) > 64800 Then Exit For
				'If GetAlbumArt(Q(I,4), "C:\Program Files\HouseBot\Config\Themes\Sonos Images\" & ZoneName & "_Queue_" & (I + 1) & ".jpg") <> 200 Then
			Next
		End If
		
		UpdateQueue = "False"
		Call SetPropertyValue(".ZPQueue", CurrentQueue)
		Call SetPropertyValue(".ZPQueueSize", CurrentQueueSize)
		Call SetPropertyValue(".ZPQueuePosition", CurrentQueuePosition)
		Call SetPropertyValue(".ZPUpdateQueue", "False")
		If Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Or Mid(CurrentURI, 1, 4) = "mms:" Then
			Call SetPropertyValue(".ZPQueuePosSize", "")
		Else
			Call SetPropertyValue(".ZPQueuePosSize", "[" & CurrentQueuePosition & " / " & CurrentQueueSize & "]")
		End If
		If SearchType = "Q:0" Then
			Call SetPropertyValue(".ZPSearchString", "UpdateQueue")
		End If
	End If
	
	If Err <> 0 Then
		TraceMessage("UpdateQueue Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
End Sub	
'==========================================================================================================
' Obtain ZonePlayer Transport and Rendering Values
'==========================================================================================================		
Private Sub GetZonePlayerStatePassively()
	On Error Resume Next
	Dim Status
	Dim NowPlayingURL
	Dim NextPlayingURL
	Dim R
	Dim Rendering
	Dim Transport
	
	Status = ZonePlayer.CheckCallbackStatus
	
	If Status(0) = True Then
		Transport = ZonePlayer.AVTransportCallback
		SavedURI = CurrentURI
		CurrentURI = Transport(10)
		CurrentTransportState = CStr(Transport(0))
		CurrentQueueSize = CStr(Transport(3))
		
		Call ZonePlayer.ResetCallbackStatus(0)
		
		If Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Or Mid(CurrentURI, 1, 4) = "mms:" Then
			
			Call SetPropertyValue(".ZPNextTrack", "")
			Call SetPropertyValue(".ZPDisplay1", "Source")
			Call SetPropertyValue(".ZPDisplay2", "Station")
			Call SetPropertyValue(".ZPDisplay3", "Information")
			Call SetPropertyValue(".ZPDisplay4", "")
		Else
			If Transport(8) <> "" Then
				Call SetPropertyValue(".ZPNextTrack", Transport(8) & " - " & Transport(7))
			Else
				Call SetPropertyValue(".ZPNextTrack", "")
			End If
			Call SetPropertyValue(".ZPDisplay1", "Track")
			Call SetPropertyValue(".ZPDisplay2", "Artist")
			Call SetPropertyValue(".ZPDisplay3", "Album")
			Call SetPropertyValue(".ZPDisplay4", "Next:")
		End If
	
		If Transport(11) <> "" Then
			If GetAlbumArt(Transport(11), NowPlayingPath) = 200 Then
				Call SetPropertyValue(".ZPNowPlayingPath", NowPlayingPath)
			Else
				Call SetPropertyValue(".ZPNowPlayingPath", NoAlbumArt)
			End If		
		Else
			If Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Then
				Call SetPropertyValue(".ZPNowPlayingPath", RadioArt)
			Else
				Call SetPropertyValue(".ZPNowPlayingPath", NoAlbumArt)
			End If
		End If
		
		If Transport(12) <> "" Then
			If GetAlbumArt(Transport(12), NextPlayingPath) = 200 Then
				Call SetPropertyValue(".ZPNextPlayingPath", NextPlayingPath)
			Else
				Call SetPropertyValue(".ZPNextPlayingPath", NoAlbumArt)
			End If
		Else
			Call SetPropertyValue(".ZPNextPlayingPath", TransparentImage)
		End If
		
		TransportUpdate = True
		CallbackTime = Now
		
		Call GetTrackInfo()
		Call UpdateGroupCoordinator()
	End If
	
	If Err <> 0 Then
		TraceMessage("CallbackStatus0 Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	
	If Status(1) = True Then	
		Rendering = ZonePlayer.RenderingCallback
		CurrentVolumeLevel = CStr(Rendering(0))
		CurrentTrebleLevel = CStr(Rendering(6))
		CurrentBassLevel = CStr(Rendering(7))
		CurrentMuteStatus = CStr(CBool(Rendering(3)))
		CurrentLoudnessStatus = CStr(CBool(Rendering(8)))
		
		Call ZonePlayer.ResetCallbackStatus(1)
		
		RenderingUpdate = True
		CallbackTime = Now
		
		Call GetTrackInfo()
		Call UpdateGroupCoordinator()
	End If
	
	If Err <> 0 Then
		TraceMessage("CallbackStatus1 Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub

Private Function GetAlbumArt(ByVal strURL, ByVal strSavePath)
	On Error Resume Next
	Dim oXMLHTTP
	Dim oADOStream
	Dim oFSO
	
	Set oXMLHTTP = CreateObject("Microsoft.XMLHTTP")
	Set oADOStream = CreateObject("ADODB.Stream")
	Set oFSO = Createobject("Scripting.FileSystemObject")
	
	oXMLHTTP.open "GET", strURL, false
	oXMLHTTP.send()
	
	GetAlbumArt = oXMLHTTP.Status
			
	If GetAlbumArt = 200 Then
		oADOStream.Open
		oADOStream.Type = 1
 		oADOStream.Write oXMLHTTP.ResponseBody
		oADOStream.Position = 0
		If oFSO.Fileexists(strSavePath) Then oFSO.DeleteFile strSavePath
 		oADOStream.SaveToFile strSavePath
 		oADOStream.Close
 	End If
 	
 	Set oXMLHTTP = Nothing
	Set oADOStream = Nothing
	Set oFSO = Nothing
	
	If Err <> 0 Then
		TraceMessage("GetAlbumArt Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
 End Function

Private Sub GetZonePlayerStateActively()
	On Error Resume Next
	CurrentTransportState = CStr(ZonePlayer.GetTransportState)
	Call SetPropertyValue(".ZPTransportState", CurrentTransportState)
	CurrentVolumeLevel = CStr(ZonePlayer.GetVolumeLevel("Master"))
	Call SetPropertyValue(".ZPVolume", CurrentVolumeLevel)
	CurrentTrebleLevel = CStr(ZonePlayer.GetTrebleLevel)
	Call SetPropertyValue(".ZPTreble", CurrentTrebleLevel)
	CurrentBassLevel = CStr(ZonePlayer.GetBassLevel)
	Call SetPropertyValue(".ZPBass", CurrentBassLevel)
	CurrentMuteStatus = CStr(ZonePlayer.GetMuteState("Master"))
	CurrentPlaymode = CStr(ZonePlayer.GetPlayMode)
	Call SetPropertyValue(".ZPPlayMode", CurrentPlaymode)
	If CurrentMuteStatus = "True" Then
		Call SetPropertyValue(".ZPMuteStatus", 1)
	Else
		Call SetPropertyValue(".ZPMuteStatus", 0)
	End If
	CurrentLoudnessStatus = CStr(ZonePlayer.GetLoudnessState("Master"))
	If CurrentLoudnessStatus = "True" Then
		Call SetPropertyValue(".ZPLoudnessStatus", 1)
	Else
		Call SetPropertyValue(".ZPLoudnessStatus", 0)
	End If
	
	If Err <> 0 Then
		TraceMessage("GetZonePlayerStateActively Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub
'==========================================================================================================
' Obtain ZonePlayer Albumt/Artist/Track Values
'==========================================================================================================
Private Sub GetTrackInfo()
	On Error Resume Next
	Dim TrackInfo
	TrackInfo = ZonePlayer.GetCurrentTrackInfo
	If VarType(TrackInfo) <> 8204 Then Exit Sub
	CurrentTrack = CStr(TrackInfo(2))
	If CurrentTrack = "" Then CurrentTrack = "[no music]"
	CurrentArtist = CStr(TrackInfo(0))
	CurrentAlbum = CStr(TrackInfo(1))
	CurrentTrackPosition = CStr(TrackInfo(5))
	CurrentTrackLength = CStr(TrackInfo(6))
	If CurrentTrackLength = "00:00" Then
		CurrentTrackLength = ""
		CurrentTrackPosition = ""
	End If
	CurrentQueuePosition = CStr(TrackInfo(7))
	If IsNumeric(CurrentQueuePosition) = False Then CurrentQueuePosition = "0"
	CurrentTrackPercent = CStr(TrackInfo(8))
	If GetPropertyValue(".ZPCurrentTrack") <> CurrentTrack Then
		Call SetPropertyValue(".ZPCurrentArtist", CurrentArtist)
		Call SetPropertyValue(".ZPCurrentAlbum", CurrentAlbum)
		Call SetPropertyValue(".ZPCurrentTrack", CurrentTrack)
		Call SetPropertyValue(".ZPTrackLength", CurrentTrackLength)
	End If
	If GetPropertyValue(".ZPTrackPosition") <> CurrentTrackPosition Then
		Call SetPropertyValue(".ZPTrackPosition", CurrentTrackPosition)
		Call SetPropertyValue(".ZPTrackPercent", CurrentTrackPercent)
	End If
	If Mid(CurrentURI, 1, 17) = "x-rincon-mp3radio" Or Mid(CurrentURI, 1, 4) = "mms:" Then
		Call SetPropertyValue(".ZPTrackPosLen", "")
	Else
		If CurrentTrackLength = "" Then
			Call SetPropertyValue(".ZPTrackPosLen", "")
		Else
			Call SetPropertyValue(".ZPTrackPosLen", CurrentTrackPosition & " / " & CurrentTrackLength)
		End If
	End If
	CurrentPlaymode = CStr(ZonePlayer.GetPlayMode)
	If SavedPlaymode <> CurrentPlaymode Then
		Select Case CurrentPlayMode
			Case "Normal"
				Call SetPropertyValue(".ZPShuffle", 0)
				SavedShuffle = 0
				Call SetPropertyValue(".ZPRepeat", 0)
				SavedRepeat = 0
			Case "Shuffle"
				Call SetPropertyValue(".ZPShuffle", 1)
				SavedShuffle = 1
				Call SetPropertyValue(".ZPRepeat", 1)
				SavedRepeat = 1
			Case "Repeat All"
				Call SetPropertyValue(".ZPShuffle", 0)
				SavedShuffle = 0
				Call SetPropertyValue(".ZPRepeat", 1)
				SavedRepeat = 1
			Case "Shuffle No Repeat"
				Call SetPropertyValue(".ZPShuffle", 1)
				SavedShuffle = 1
				Call SetPropertyValue(".ZPRepeat", 0)
				SavedRepeat = 0
		End Select	
		Call SetPropertyValue(".ZPPlayMode", CurrentPlaymode)
		SavedPlaymode = CurrentPlaymode
	End If
	If Err <> 0 Then
		TraceMessage("GetTrackInfo Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
	CurrentQueueSize = ZonePlayer.GetQueueSize
End Sub
'==========================================================================================================
' Compare the New HouseBot Device Values with the Saved Values
'==========================================================================================================
Private Sub CompareNewToSaved()
	On Error Resume Next
	If NewTransportState <> SavedTransportState Then
		If GetPropertyValue(".ZPLinked") = 0 Then
			Call ZonePlayer.SetTransportState(CStr(NewTransportState))
			If CStr(NewTransportState) = "Next" or CStr(NewTransportState) = "Previous" Then
				Call SetPropertyValue(".ZPTransportState", "Play")
			End If
			SavedTransportState = NewTransportState
			Update(0) = True
		Else
			Call SetPropertyValue(DeviceCoordinator & ".ZPTransportState", NewTransportState)
			SavedTransportState = NewTransportState
			Update(0) = True
		End If
	End If
	If NewShuffle <> SavedShuffle Or NewRepeat <> SavedRepeat Then
		Select Case NewShuffle
			Case 0
				If NewRepeat = 0 Then
					Call ZonePlayer.PlayMode_Normal
				Else
					Call ZonePlayer.PlayMode_RepeatAll
				End If
			Case 1
				If NewRepeat = 1 Then
					Call ZonePlayer.PlayMode_Shuffle
				Else
					Call ZonePlayer.PlayMode_ShuffleNoRepeat
				End If
		End Select
		SavedShuffle = NewShuffle
		SavedRepeat = NewRepeat
	End If
	If NewPlayMode <> SavedPlayMode Then
		If NewPlayMode = "Normal" Then
			Call ZonePlayer.PlayMode_Normal
		ElseIf NewPlayMode = "Repeat All" Then
			Call ZonePlayer.PlayMode_RepeatAll
		ElseIf NewPlayMode = "Shuffle" Then
			Call ZonePlayer.PlayMode_Shuffle
		ElseIf NewPlayMode = "Shuffle No Repeat" Then
			Call ZonePlayer.PlayMode_ShuffleNoRepeat
		End If
		Select Case NewPlayMode
			Case "Normal"
				Call SetPropertyValue(".ZPShuffle", 0)
				SavedShuffle = 0
				Call SetPropertyValue(".ZPRepeat", 0)
				SavedRepeat = 0
			Case "Shuffle"
				Call SetPropertyValue(".ZPShuffle", 1)
				SavedShuffle = 1
				Call SetPropertyValue(".ZPRepeat", 1)
				SavedRepeat = 1
			Case "Repeat All"
				Call SetPropertyValue(".ZPShuffle", 0)
				SavedShuffle = 0
				Call SetPropertyValue(".ZPRepeat", 1)
				SavedRepeat = 1
			Case "Shuffle No Repeat"
				Call SetPropertyValue(".ZPShuffle", 1)
				SavedShuffle = 1
				Call SetPropertyValue(".ZPRepeat", 0)
				SavedRepeat = 0
		End Select
		SavedPlayMode = NewPlayMode
		Call CloseRemotePanel("PlayMode-" & ZoneName)
		Update(1) = True
	End If
	If NewVolumeLevel <> SavedVolumeLevel Then
		Call ZonePlayer.SetVolumeLevel("Master", CInt(NewVolumeLevel))
		SavedVolumeLevel = NewVolumeLevel
		Update(2) = True
	End If
	If NewTrebleLevel <> SavedTrebleLevel Then
		Call ZonePlayer.SetTrebleLevel(CInt(NewTrebleLevel))
		SavedTrebleLevel = NewTrebleLevel
		Update(3) = True
	End If
	If NewBassLevel <> SavedBassLevel Then
		Call ZonePlayer.SetBassLevel(CInt(NewBassLevel))
		SavedBassLevel = NewBassLevel
		Update(4) = True
	End If
	If NewMuteStatus <> SavedMuteStatus Then
		If NewMuteStatus = "True" Then
			Call ZonePlayer.SetMuteState("Master", True)
		Else
			Call ZonePlayer.SetMuteState("Master", False)
		End If
		SavedMuteStatus = NewMuteStatus
		Update(5) = True
	End If
	If NewLoudnessStatus <> SavedLoudnessStatus Then
		If NewLoudnessStatus = "True" Then
			Call ZonePlayer.SetLoudnessState("Master", True)
		Else
			Call ZonePlayer.SetLoudnessState("Master", False)
		End If
		SavedLoudnessStatus = NewLoudnessStatus
		Update(6) = True
	End If
	
	If Err <> 0 Then
		TraceMessage("CompareNewToSaved Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub
'==========================================================================================================
' Browse the Media Collection
'==========================================================================================================
Private Sub BrowseMedia()
	On Error Resume Next
	Dim Result
	Dim Title
	Dim Title2
	MusicMenuTitle(0) = "Music Menu"
	BrowseReturn = "100"						
	If Browse = "Forward" Then
		If IsNumeric(BrowseSelection) Then
			If CInt(BrowseSelection) = -1 Then
				Call SetPropertyValue(".ZPBrowse", "Hold")
				Exit Sub
			End If
		End If
		If Mid(BrowseSelection, 1, 1) = "x" or Mid(BrowseSelection, 1, 1) = "f" Then	
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call OpenRemotePanel("AddSong-" & ZoneName)							
			Exit Sub
		Else	
			SavedBrowseIndex(BrowseDepth) = BrowseIndex
			SavedBrowseSelection(BrowseDepth + 1) = BrowseSelection			
			BrowseDepth = BrowseDepth + 1						
			Result = ZonePlayer.Browse(BrowseSelection, BrowseReturn, BrowseIndex)
			If VarType(Result) <> 8204 Then Exit Sub
			BrowseList(BrowseDepth) = Result(0)
			BrowseTotalItems(BrowseDepth) = Result(1)
			
			If Mid(BrowseSelection, 1, 1) = "A" Then
				Title = Split(BrowseSelection, ":")
				If InStr(Title(1), "/") > 0 Then
					Title2 = Split(Title(1), "/")
					Dim H
					For H = 0 to UBound(Title2)
					MusicMenuTitle(H+1) = unescape(Title2(H))
					If H = 0 Then
						If MusicMenuTitle(1) = "ALBUM" Then MusicMenuTitle(H+1) = "Albums"
						If MusicMenuTitle(1) = "ARTIST" Then MusicMenuTitle(H+1) = "Artists"
						If MusicMenuTitle(1) = "TRACKS" Then MusicMenuTitle(H+1) = "Tracks"
						If MusicMenuTitle(1) = "GENRE" Then MusicMenuTitle(H+1) = "Genre"
						If MusicMenuTitle(1) = "COMPOSERS" Then MusicMenuTitle(H+1) = "Composers"
					End If
					Next
					Call SetPropertyValue(".ZPMusicMenuTitle", MusicMenuTitle(BrowseDepth))
				Else
					MusicMenuTitle(1) = Title(1)
					If MusicMenuTitle(1) = "ALBUM" Then MusicMenuTitle(H+1) = "Albums"
					If MusicMenuTitle(1) = "ARTIST" Then MusicMenuTitle(H+1) = "Artists"
					If MusicMenuTitle(1) = "TRACKS" Then MusicMenuTitle(H+1) = "Tracks"
					If MusicMenuTitle(1) = "GENRE" Then MusicMenuTitle(H+1) = "Genre"
					If MusicMenuTitle(1) = "COMPOSERS" Then MusicMenuTitle(H+1) = "Composers"
					Call SetPropertyValue(".ZPMusicMenuTitle", MusicMenuTitle(BrowseDepth))
				End If
			ElseIf Mid(BrowseSelection, 1, 1) = "R" Then
				Title = Split(BrowseSelection, ":")
				MusicMenuTitle(1) = "Internet Radio"
				If Title(1) <> "" Then
					MusicMenuTitle(2) = RadioCatalog(CInt(Title(1)))
				End If
				Call SetPropertyValue(".ZPMusicMenuTitle", MusicMenuTitle(BrowseDepth))
			End If
			Call SetPropertyValue(".ZPBrowseTotalItems", BrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseList", BrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseIndex", BrowseIndex)
		End If
	ElseIf Browse = "Back" Then													
		If BrowseDepth > 0 Then
			BrowseDepth = BrowseDepth - 1										
			Call SetPropertyValue(".ZPBrowseTotalItems", BrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseList", BrowseList(BrowseDepth))
			BrowseIndex = SavedBrowseIndex(BrowseDepth)
			BrowseSelection = SavedBrowseSelection(BrowseDepth)
			Call SetPropertyValue(".ZPMusicMenuTitle", MusicMenuTitle(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseIndex", BrowseIndex)
			Call SetPropertyValue(".ZPBrowseSelection", BrowseSelection)
		End If
		Call SetPropertyValue(".ZPBrowse", "Hold")
	ElseIf Browse = "Up" Then				
		If CInt(BrowseIndex) > 0 Then
			BrowseIndex = CStr(CInt(BrowseIndex) - CInt(BrowseReturn))
			If CInt(BrowseIndex) < 0 Then BrowseIndex = "0"
			Result = ZonePlayer.Browse(SavedBrowseSelection(BrowseDepth), BrowseReturn, BrowseIndex)
			If VarType(Result) <> 8204 Then Exit Sub
			BrowseList(BrowseDepth) = Result(0)
			BrowseTotalItems(BrowseDepth) = Result(1)
			Call SetPropertyValue(".ZPBrowseTotalItems", BrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseList", BrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseIndex", BrowseIndex)
		End If
		Call SetPropertyValue(".ZPBrowse", "Hold")
	ElseIf Browse = "Down" Then
		If CInt(BrowseTotalItems(BrowseDepth)) < CInt(BrowseReturn) Then
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Exit Sub
		ElseIf (CInt(BrowseIndex) + CInt(BrowseReturn)) < CInt(BrowseTotalItems(BrowseDepth)) Then
			BrowseIndex = CStr(CInt(BrowseIndex) + CInt(BrowseReturn))
			Result = ZonePlayer.Browse(SavedBrowseSelection(BrowseDepth), BrowseReturn, BrowseIndex)
			If VarType(Result) <> 8204 Then Exit Sub
			BrowseList(BrowseDepth) = Result(0)
			BrowseTotalItems(BrowseDepth) = Result(1)
			Call SetPropertyValue(".ZPBrowseTotalItems", BrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseList", BrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseIndex", BrowseIndex)
		End If
		Call SetPropertyValue(".ZPBrowse", "Hold")
	End If
	BrowsePercent = 100 - Round(100 * (CInt(GetPropertyValue(".ZPBrowseIndex"))/CInt(GetPropertyValue(".ZPBrowseTotalItems"))))
	Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
	
	If Err <> 0 Then
		TraceMessage("BrowseMedia Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub

Private Sub BrowseMedia2()
	Dim Result
	
	If BrowseReturn = "" Then BrowseReturn = "100"
	If BrowseIndex = "" Then BrowseIndex = "0"
	
	If SearchType <> SavedSearchType Or SearchIndex <> "" Then
		If SearchIndex = "" Then
			BrowseIndex = "0"
		ElseIf SearchIndex = "UpdateQueue" Then
			Call SetPropertyValue(".ZPSearchString", "")
		ElseIf Len(SearchIndex) = 1 And IsNumeric(SearchIndex) = False And Mid(SearchType, 1, 1) = "A" Then
			SearchIndex = UCase(SearchIndex)
			BrowseIndex = ZonePlayer.FindPrefix(SearchType, SearchIndex)
			If CStr(BrowseIndex) = "" Then BrowseIndex = "0"
			Call SetPropertyValue(".ZPSearchString", "")
			Call CloseRemotePanel("PowerScroll-" & ZoneName)
		Else
			Call SetPropertyValue(".ZPSearchString", "")
			Exit Sub
		End If
		Result = ZonePlayer.Browse(SearchType, BrowseReturn, BrowseIndex)
		If VarType(Result) <> 8204 Then Exit Sub
		BrowseDepth = 0
		Select Case SearchType
			Case "A:ALBUM"
				MusicMenuTitle(0) = "Albums"
			Case "A:ARTIST"
				MusicMenuTitle(0) = "Artist"
			Case "A:TRACKS"
				MusicMenuTitle(0) = "Tracks"
			Case "A:GENRE"
				MusicMenuTitle(0) = "Genre"
			Case "A:COMPOSER"
				MusicMenuTitle(0) = "Composers"
			Case "A:PLAYLISTS"
				MusicMenuTitle(0) = "Imported Playlists"
			Case "R:"
				MusicMenuTitle(0) = "Internet Radio"
			Case "S:"
				MusicMenuTitle(0) = "Folders"
			Case "SQ:"
				MusicMenuTitle(0) = "Playlists"
			Case "AI:"
				MusicMenuTitle(0) = "Audio In"
			Case "Q:0"
				MusicMenuTitle(0) = "Queue"
		End Select
		SavedBrowseIndex(BrowseDepth) = BrowseIndex
		SavedBrowseSelection(BrowseDepth) = SearchType
		SavedBrowseList(BrowseDepth) = Result(0)
		SavedBrowseTotalItems(BrowseDepth) = Result(1)
		SavedSearchType = SearchType
		If CInt(SavedBrowseTotalItems(BrowseDepth)) = 0 Then
			BrowsePercent = 0
		ElseIf CInt(SavedBrowseIndex(BrowseDepth)) + CInt(BrowseReturn) > CInt(SavedBrowseTotalItems(BrowseDepth)) Then
			BrowsePercent = 0
		Else
			BrowsePercent = 100 - Round(100 * (CInt(SavedBrowseIndex(BrowseDepth))/CInt(SavedBrowseTotalItems(BrowseDepth))))
		End If
		If SearchType = "Q:0" Then
			Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
			Call SetPropertyValue(".ZPBrowseIndex", SavedBrowseIndex(BrowseDepth))			
			Call SetPropertyValue(".ZPQueueList", SavedBrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseTotalItems", SavedBrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseDepth", BrowseDepth)
		Else
			Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
			Call SetPropertyValue(".ZPBrowseIndex", SavedBrowseIndex(BrowseDepth))			
			Call SetPropertyValue(".ZPBrowseList", SavedBrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseTotalItems", SavedBrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPMusicMenuTitle", MusicMenuTitle(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseDepth", BrowseDepth)
		End If
	ElseIf Browse <> "Hold" Then
		Select Case Browse
			Case "Forward"
				BrowseIndex = "0"
				If IsNumeric(BrowseSelection) Then
					If CInt(BrowseSelection) = -1 Then
						Call SetPropertyValue(".ZPBrowse", "Hold")
						Exit Sub
					End If
				ElseIf Mid(BrowseSelection, 1, 1) = "x" or Mid(BrowseSelection, 1, 1) = "f" Then	
					Call SetPropertyValue(".ZPBrowse", "Hold")
					Call OpenRemotePanel("AddSong-" & ZoneName)							
					Exit Sub
				ElseIf BrowseSelection = "Up" Then
					Call SetPropertyValue(".ZPBrowse", "Up")
					Exit Sub
				ElseIf BrowseSelection = "Down" Then
					Call SetPropertyValue(".ZPBrowse", "Down")
					Exit Sub
				Else							
					Result = ZonePlayer.Browse(BrowseSelection, BrowseReturn, BrowseIndex)
					If VarType(Result) <> 8204 Then Exit Sub
					BrowseDepth = BrowseDepth + 1
					SavedBrowseIndex(BrowseDepth) = BrowseIndex
					SavedBrowseSelection(BrowseDepth) = BrowseSelection
					SavedBrowseList(BrowseDepth) = Result(0)
					SavedBrowseTotalItems(BrowseDepth) = Result(1)
					Select Case Mid(SearchType, 1, 1)
						Case "A"
							MusicMenuTitle(BrowseDepth) = Replace(unescape(Replace(BrowseSelection, SearchType & "/", "")), "/", " - ")
						Case "R"
							If InStr(BrowseSelection, "/") Then
								Select Case Right(BrowseSelection, 1)
									Case "0"
										MusicMenuTitle(BrowseDepth) = "Stations"
									Case "1"
										MusicMenuTitle(BrowseDepth) = "Shows"
								End Select
							Else
								MusicMenuTitle(BrowseDepth) = RadioCatalog(CInt(Replace(BrowseSelection, "R:", "")))
							End If
					End Select
				End If
			Case "Back"
				If BrowseDepth > 0 Then
					BrowseDepth = BrowseDepth - 1
					Call SetPropertyValue(".ZPMusicSelectionList", "")
				Else
					Call SetPropertyValue(".ZPMusicSelectionList", "")
					Exit Sub
				End If
			Case "Up"
				If CInt(SavedBrowseIndex(BrowseDepth)) > 0 Then
					SavedBrowseIndex(BrowseDepth) = CStr(CInt(SavedBrowseIndex(BrowseDepth)) - CInt(BrowseReturn))
					If CInt(SavedBrowseIndex(BrowseDepth)) < 0 Then SavedBrowseIndex(BrowseDepth) = "0"
					Result = ZonePlayer.Browse(SavedBrowseSelection(BrowseDepth), BrowseReturn, SavedBrowseIndex(BrowseDepth))
					If VarType(Result) <> 8204 Then Exit Sub
					SavedBrowseList(BrowseDepth) = Result(0)
					SavedBrowseTotalItems(BrowseDepth) = Result(1)
				Else
					Exit Sub
				End If
			Case "Down"
				If CInt(SavedBrowseTotalItems(BrowseDepth)) < CInt(BrowseReturn) Then
					Call SetPropertyValue(".ZPBrowse", "Hold")
					Exit Sub
				ElseIf (CInt(SavedBrowseIndex(BrowseDepth)) + CInt(BrowseReturn)) < CInt(SavedBrowseTotalItems(BrowseDepth)) Then
					SavedBrowseIndex(BrowseDepth) = CStr(CInt(SavedBrowseIndex(BrowseDepth)) + CInt(BrowseReturn))
					Result = ZonePlayer.Browse(SavedBrowseSelection(BrowseDepth), BrowseReturn, SavedBrowseIndex(BrowseDepth))
					If VarType(Result) <> 8204 Then Exit Sub
					SavedBrowseList(BrowseDepth) = Result(0)
					SavedBrowseTotalItems(BrowseDepth) = Result(1)
				Else
					Exit Sub
				End If
		End Select
		If CInt(SavedBrowseTotalItems(BrowseDepth)) = 0 Then
			BrowsePercent = 0
		ElseIf CInt(SavedBrowseIndex(BrowseDepth)) + CInt(BrowseReturn) > CInt(SavedBrowseTotalItems(BrowseDepth)) Then
			BrowsePercent = 0
		Else
			BrowsePercent = 100 - Round(100 * (CInt(SavedBrowseIndex(BrowseDepth))/CInt(SavedBrowseTotalItems(BrowseDepth))))
		End If
		If SearchType = "Q:0" Then
			Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
			Call SetPropertyValue(".ZPBrowseIndex", SavedBrowseIndex(BrowseDepth))			
			Call SetPropertyValue(".ZPQueueList", SavedBrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseTotalItems", SavedBrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseDepth", BrowseDepth)
		ElseIf BrowseDepth = 1 and SearchType = "A:ALBUM" Then
			Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
			Call SetPropertyValue(".ZPBrowseIndex", SavedBrowseIndex(BrowseDepth))			
			Call SetPropertyValue(".ZPMusicSelectionList", SavedBrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseTotalItems", SavedBrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPMusicSelectionTitle", MusicMenuTitle(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseDepth", BrowseDepth)
		ElseIf BrowseDepth = 2 and SearchType = "A:ARTIST" Then
			Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
			Call SetPropertyValue(".ZPBrowseIndex", SavedBrowseIndex(BrowseDepth))			
			Call SetPropertyValue(".ZPMusicSelectionList", SavedBrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseTotalItems", SavedBrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPMusicSelectionTitle", MusicMenuTitle(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseDepth", BrowseDepth)
		ElseIf BrowseDepth = 3 and SearchType = "A:GENRE" Then
			Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
			Call SetPropertyValue(".ZPBrowseIndex", SavedBrowseIndex(BrowseDepth))			
			Call SetPropertyValue(".ZPMusicSelectionList", SavedBrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseTotalItems", SavedBrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPMusicSelectionTitle", MusicMenuTitle(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseDepth", BrowseDepth)
		Else
			Call SetPropertyValue(".ZPBrowsePercent", BrowsePercent)
			Call SetPropertyValue(".ZPBrowseIndex", SavedBrowseIndex(BrowseDepth))			
			Call SetPropertyValue(".ZPBrowseList", SavedBrowseList(BrowseDepth))
			Call SetPropertyValue(".ZPBrowseTotalItems", SavedBrowseTotalItems(BrowseDepth))
			Call SetPropertyValue(".ZPMusicMenuTitle", MusicMenuTitle(BrowseDepth))
			Call SetPropertyValue(".ZPBrowse", "Hold")
			Call SetPropertyValue(".ZPBrowseDepth", BrowseDepth)
			Call SetPropertyValue(".ZPMusicSelectionList", "" & vbTab & "")
			Call SetPropertyValue(".ZPMusicSelectionTitle", "")
		End If
	End If
	
	If Err <> 0 Then
		TraceMessage("BrowseMedia Error: " & Err.Description & " (" & Err.Number & ")")
		Err.Clear
	End If
End Sub
	
Private Sub StandardizeLength(ByRef strBase, ByRef intLength)
    If Len(strBase) > intLength Then
        Dim L
        L = intLength - 4
        strBase = Mid(strBase, 1, L) & "..."
        Exit Sub
    End If
    Do Until Len(strBase) = intLength
        strBase = strBase & " "
    Loop
End Sub